use serde::{Serialize, Deserialize};

#[derive(Serialize)]
#[serde(rename_all = "camelCase")]
pub struct Room {
    pub room_number: i32,
    pub floor: i32,
    pub size: i32,
    pub room_type: String,
    pub beds: i32,
    pub accessibility: bool,
}

#[derive(Serialize, Deserialize, Clone)]
#[serde(rename_all = "camelCase")]
pub struct  Address {
    pub street: String,
    pub house_number: i32,
    pub postal_code: i32,
    pub city: String,
    pub country: String,
}

#[derive(Serialize)]
#[serde(rename_all = "camelCase")]
pub struct  Person {
    pub id: i32,
    pub first_name: String,
    pub last_name: String,
    pub age: i32,
    pub address: Address,
}

#[derive(Serialize)]
#[serde(rename_all = "camelCase")]
pub struct  Clerk {
    pub staff_number: i32,
    pub salary: i32,
    pub person_data: Person,
}

#[derive(Serialize)]
#[serde(rename_all = "camelCase")]
pub struct  Client {
    pub id: i32,
    pub bank_details: String,
    pub person_data: Person,
}

#[derive(Serialize)]
#[serde(rename_all = "camelCase")]
pub struct Booking {
    pub id: i32,
    pub arrival_date: String,
    pub depature_date: String,
    pub cost: i32,
    pub pension_type: String,
    pub client_id: i32,
}

#[derive(Serialize)]
#[serde(rename_all = "camelCase")]
pub struct CleaningPlan {
    pub id: i32,
    pub date: String,
    pub duration: i32,
    pub room_id: i32,
}

#[derive(Deserialize)]
pub struct  BookingQuery {
    pub months: Option<i32>,
}

#[derive(Deserialize)]
pub struct  ClerkMonthsQuery {
    pub months: i32,
}

#[derive(Deserialize)]
pub struct  PersonQuery {
    pub firstname: Option<String>,
    pub lastname: Option<String>,
}

#[derive(Deserialize, Clone)]
#[serde(rename_all = "camelCase")]
pub struct BookingBody {
    pub arrival_date: String,
    pub depature_date: String,
    pub cost: i32,
    pub pension_type: String,
    pub client_id: i32,
}

#[derive(Deserialize, Clone)]
#[serde(rename_all = "camelCase")]
pub struct PersonBody {
    pub first_name: String,
    pub last_name: String,
    pub age: i32,
    pub address: Address,
}

#[derive(Deserialize, Clone)]
#[serde(rename_all = "camelCase")]
pub struct ClientBody {
    pub bank_details: String,
}