use axum::{extract::{Extension, Query, Path}, response::IntoResponse, Json, routing::BoxRoute, Router, handler::get};
use hyper::StatusCode;

use crate::{error::ServiceError, Service, model::ClerkMonthsQuery};

async fn clerks(
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let clerks = service.get_clerks().await?;
    Ok(Json(clerks))
}

async fn clerk(
    Path(id): Path<i32>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let clerk = service.get_clerk(id).await?
        .ok_or_else(|| ServiceError::ErrorResponse(StatusCode::NOT_FOUND, Some("clerk not found".to_string())))?;
    Ok(Json(clerk))
}

async fn most_valuable_clerk(
    Query(query): Query<ClerkMonthsQuery>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let clerk = service.get_most_valuable_clerk(query.months).await?
        .ok_or_else(|| ServiceError::ErrorResponse(StatusCode::NOT_FOUND, Some("clerk not found".to_string())))?;
    Ok(Json(clerk))
}

pub fn routes() -> Router<BoxRoute> {
    Router::new()
        .route("/", get(clerks))
        .route("/:id", get(clerk))
        .route("/mostValuable", get(most_valuable_clerk))
        .boxed()
}