use axum::{Router, routing::BoxRoute, extract::{Extension, Path}, response::IntoResponse, Json, handler::get};
use hyper::StatusCode;

use crate::{Service, error::ServiceError, model::{ClientBody, Client}};

async fn clients(
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let clients = service.get_clients().await?;
    Ok(Json(clients))
}

async fn client(
    Path(id): Path<i32>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let client = service.get_client(id).await?
        .ok_or_else(|| ServiceError::ErrorResponse(StatusCode::NOT_FOUND, Some("client not found".to_string())))?;
    Ok(Json(client))
}

async fn set_client(
    Json(payload): Json<ClientBody>,
    Path(id): Path<i32>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let person = service.get_person(id).await?
        .ok_or_else(|| ServiceError::ErrorResponse(StatusCode::NOT_FOUND, Some("person does not exist".to_string())))?;
    service.set_client(id, payload.clone()).await?;
    let client = Client {
        id,
        bank_details: payload.bank_details,
        person_data: person,
    };
    Ok(Json(client))
}

async fn delete_client(
    Path(id): Path<i32>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    service.delete_client(id).await?;
    Ok(StatusCode::NO_CONTENT)
}

pub fn routes() -> Router<BoxRoute> {
    Router::new()
        .route("/", get(clients))
        .route("/:id", get(client).put(set_client).delete(delete_client))
        .boxed()
}