use axum::{Router, routing::BoxRoute, extract::{Path, Extension, Query}, response::IntoResponse, Json, handler::get};
use hyper::StatusCode;

use crate::{Service, error::ServiceError, model::{PersonBody, Person, PersonQuery}};

async fn persons(
    Query(query): Query<PersonQuery>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let persons = service.get_persons(query).await?;
    Ok(Json(persons))
}

async fn person(
    Path(id): Path<i32>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let person = service.get_person(id).await?
        .ok_or_else(|| ServiceError::ErrorResponse(StatusCode::NOT_FOUND, Some("person not found".to_string())))?;
    Ok(Json(person))
}

async fn add_person(
    Json(payload): Json<PersonBody>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let id = service.add_person(payload.clone()).await?;
    let person = Person {
        id,
        first_name: payload.first_name,
        last_name: payload.last_name,
        age: payload.age,
        address: payload.address,
    };
    Ok((StatusCode::CREATED, Json(person)))
}

pub fn routes() -> Router<BoxRoute> {
    Router::new()
        .route("/", get(persons).post(add_person))
        .route("/:id", get(person))
        .boxed()
}