use axum::{extract::{Extension, Path}, response::IntoResponse, Json, Router, routing::BoxRoute, handler::get};
use hyper::StatusCode;

use crate::{error::ServiceError, Service};

async fn rooms(
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let rooms = service.get_rooms().await?;
    Ok(Json(rooms))
}

async fn room(
    Path(id): Path<i32>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let room = service.get_room(id).await?
        .ok_or_else(|| ServiceError::ErrorResponse(StatusCode::NOT_FOUND, Some("room not found".to_string())))?;
    Ok(Json(room))
}

async fn cleaning_plans(
    Path(id): Path<i32>,
    Extension(service): Extension<Service>,
) -> Result<impl IntoResponse, ServiceError> {
    let cleaning_plans = service.get_room_cleaning_plans(id).await?;
    Ok(Json(cleaning_plans))
}

pub fn routes() -> Router<BoxRoute> {
    Router::new()
        .route("/", get(rooms))
        .route("/:id", get(room))
        .route("/:id/cleaningPlans", get(cleaning_plans))
        .boxed()
}