use sibyl::Row;

use crate::{ServiceInner, model::Clerk, error::ServiceError};

impl ServiceInner {

    pub async fn get_clerks(&self) -> Result<Vec<Clerk>, ServiceError> {
        let session = self.pool.get_session().await?;
        let stmt = session.prepare("
            SELECT
                c.staff,
                c.salary,
                p.personId,
                p.name,
                p.lastName,
                p.age,
                ad.street,
                ad.houseNumber,
                ad.postalCode,
                ad.city,
                ad.country
            FROM Clerk c
            INNER JOIN Person p ON p.personId = c.personId
            INNER JOIN Address ad ON p.addressId = ad.addressId
        ").await?;

        let rows = stmt.query("").await?;

        let mut clerks: Vec<Clerk> = vec![];

        while let Some(row) = rows.next().await? {
            clerks.push(Self::map_clerk_row(row)?);
        }

        Ok(clerks)
    }

    pub async fn get_clerk(&self, id: i32) -> Result<Option<Clerk>, ServiceError> {
        let session = self.pool.get_session().await?;
        let stmt = session.prepare("
            SELECT
                c.staff,
                c.salary,
                p.personId,
                p.name,
                p.lastName,
                p.age,
                ad.street,
                ad.houseNumber,
                ad.postalCode,
                ad.city,
                ad.country
            FROM Clerk c
            INNER JOIN Person p ON p.personId = c.personId
            INNER JOIN Address ad ON p.addressId = ad.addressId
            WHERE c.staff = :ID
        ").await?;

        let row = stmt.query_single(id).await?;

        Ok(match row {
            Some(row) => {
                Some(Self::map_clerk_row(row)?)
            },
            None => None,
        })
    }

    pub async fn get_most_valuable_clerk(&self, months: i32) -> Result<Option<Clerk>, ServiceError> {
        let session = self.pool.get_session().await?;
        let stmt = session.prepare("
            SELECT
                c.staff,
                c.salary,
                p.personId,
                p.name,
                p.lastName,
                p.age,
                ad.street,
                ad.houseNumber,
                ad.postalCode,
                ad.city,
                ad.country
            FROM Clerk c
            INNER JOIN Person p ON p.personId = c.personId
            INNER JOIN Address ad ON p.addressId = ad.addressId
            WHERE p.personId = mostValuableClerk(:MONTHS)
        ").await?;

        let row = stmt.query_single(months).await?;

        Ok(match row {
            Some(row) => {
                Some(Self::map_clerk_row(row)?)
            },
            None => None,
        })
    }

    pub fn map_clerk_row(row: Row) -> Result<Clerk, ServiceError> {
        Ok(Clerk {
            staff_number: row.get(0)?,
            salary: row.get(1)?,
            person_data: Self::map_person_row(row, 2)?,
        })
    }

}