use sibyl::Row;

use crate::{ServiceInner, model::{Room, CleaningPlan}, error::ServiceError};

impl ServiceInner {

    pub async fn get_rooms(&self) -> Result<Vec<Room>, ServiceError> {
        let session = self.pool.get_session().await?;
        let stmt = session.prepare("
            SELECT
                roomNumber,
                floor,
                roomTyp,
                \"size\",
                accessibility,
                beds
            FROM room
        ").await?;

        let rows = stmt.query("").await?;

        let mut rooms: Vec<Room> = vec![];

        while let Some(row) = rows.next().await? {
            rooms.push(Self::map_room_row(row)?);
        }
        
        Ok(rooms)
    }

    pub async fn get_room(&self, id: i32) -> Result<Option<Room>, ServiceError> {
        let session = self.pool.get_session().await?;
        let stmt = session.prepare("
            SELECT
                roomNumber,
                floor,
                roomTyp,
                \"size\",
                accessibility,
                beds
            FROM room WHERE roomNumber = :ID
        ").await?;

        let row = stmt.query_single(id).await?;
        
        Ok(match row {
            Some(row) => {
                Some(Self::map_room_row(row)?)
            },
            None => None,
        })
    }

    pub async fn get_room_cleaning_plans(&self, roomid: i32) -> Result<Vec<CleaningPlan>, ServiceError> {
        let session = self.pool.get_session().await?;
        let stmt = session.prepare("
            SELECT
                cleaningPlanId,
                TO_CHAR(\"date\", 'YYYY-MM-DD'), 
                duration
            FROM CleaningPlan
            WHERE roomNumber = :ROOMID
        ").await?;

        let rows = stmt.query(roomid).await?;

        let mut cleaning_plans: Vec<CleaningPlan> = vec![];

        while let Some(row) = rows.next().await? {
            let cleaning_plan = CleaningPlan {
                id: row.get(0)?,
                date: row.get(1)?,
                duration: row.get(2)?,
                room_id: roomid,
            };
            cleaning_plans.push(cleaning_plan);
        }

        Ok(cleaning_plans)
    }

    pub fn map_room_row(row: Row) -> Result<Room, ServiceError> {
        let acc: i32 = row.get(5)?;
        Ok(Room {
            room_number: row.get(0)?,
            floor: row.get(1)?,
            size: row.get(3)?,
            room_type: row.get(2)?,
            beds: row.get(4)?,
            accessibility: acc != 0,
        })
    }

}